@@ -1,6 +1,6 @@
 use std::any::TypeId;
 use std::cell::{RefCell, UnsafeCell};
-use std::collections::HashMap;
+use std::collections::{HashMap, HashSet};
 use std::ffi::CString;
 use std::marker::PhantomData;
 use std::os::raw::{c_char, c_int, c_void};
@@ -18,8 +18,9 @@ use crate::table::Table;
 use crate::thread::Thread;
 use crate::types::{
     Callback, HookCallback, Integer, LightUserData, LuaRef, MaybeSend, Number, RegistryKey,
+    UserDataCell,
 };
-use crate::userdata::{AnyUserData, MetaMethod, UserData, UserDataMethods};
+use crate::userdata::{AnyUserData, MetaMethod, UserData, UserDataMethods, UserDataWrapped};
 use crate::util::{
     assert_stack, callback_error, check_stack, get_gc_userdata, get_main_state,
     get_meta_gc_userdata, get_wrapped_error, init_error_registry, init_gc_metatable_for,
@@ -39,6 +40,9 @@ use {
     futures_util::future::{self, TryFutureExt},
 };
 
+#[cfg(feature = "serialize")]
+use {crate::util::get_destructed_userdata_metatable, serde::Serialize};
+
 /// Top level Lua struct which holds the Lua state itself.
 pub struct Lua {
     pub(crate) state: *mut ffi::lua_State,
@@ -53,8 +57,10 @@ pub struct Lua {
 // Data associated with the lua_State.
 struct ExtraData {
     registered_userdata: HashMap<TypeId, c_int>,
+    registered_userdata_mt: HashSet<isize>,
     registry_unref_list: Arc<Mutex<Option<Vec<c_int>>>>,
 
+    libs: StdLib,
     mem_info: *mut MemoryInfo,
 
     ref_thread: *mut ffi::lua_State,
@@ -94,6 +100,7 @@ pub(crate) static EXTRA_REGISTRY_KEY: u8 = 0;
 
 /// Requires `feature = "send"`
 #[cfg(feature = "send")]
+#[cfg_attr(docsrs, doc(cfg(feature = "send")))]
 unsafe impl Send for Lua {}
 
 impl Drop for Lua {
@@ -166,7 +173,9 @@ impl Lua {
 
         let mut lua = unsafe { Self::unsafe_new_with(libs) };
 
-        mlua_expect!(lua.disable_c_modules(), "Error during disabling C modules");
+        if libs.contains(StdLib::PACKAGE) {
+            mlua_expect!(lua.disable_c_modules(), "Error during disabling C modules");
+        }
         lua.safe = true;
 
         Ok(lua)
@@ -256,7 +265,7 @@ impl Lua {
         lua.ephemeral = false;
         #[cfg(any(feature = "lua54", feature = "lua53", feature = "lua52"))]
         {
-            lua.extra.lock().unwrap().mem_info = mem_info;
+            mlua_expect!(lua.extra.lock(), "extra is poisoned").mem_info = mem_info;
         }
 
         mlua_expect!(
@@ -265,6 +274,7 @@ impl Lua {
             }),
             "Error during loading standard libraries"
         );
+        mlua_expect!(lua.extra.lock(), "extra is poisoned").libs |= libs;
 
         lua
     }
@@ -294,6 +304,10 @@ impl Lua {
                     init_gc_metatable_for::<Waker>(state, None);
                 }
 
+                // Init serde metatables
+                #[cfg(feature = "serialize")]
+                crate::serde::init_metatables(state);
+
                 // Create ref stack thread and place it in the registry to prevent it from being garbage
                 // collected.
 
@@ -308,8 +322,10 @@ impl Lua {
 
         let extra = Arc::new(Mutex::new(ExtraData {
             registered_userdata: HashMap::new(),
+            registered_userdata_mt: HashSet::new(),
             registry_unref_list: Arc::new(Mutex::new(Some(Vec::new()))),
             ref_thread,
+            libs: StdLib::NONE,
             mem_info: ptr::null_mut(),
             // We need 1 extra stack space to move values in and out of the ref stack.
             ref_stack_size: ffi::LUA_MINSTACK - 1,
@@ -370,11 +386,20 @@ impl Lua {
         }
 
         let state = self.main_state.unwrap_or(self.state);
-        unsafe {
+        let res = unsafe {
             protect_lua_closure(state, 0, 0, |state| {
                 load_from_std_lib(state, libs);
             })
+        };
+
+        // If `package` library loaded into a safe lua state then disable C modules
+        let curr_libs = mlua_expect!(self.extra.lock(), "extra is poisoned").libs;
+        if self.safe && (curr_libs ^ (curr_libs | libs)).contains(StdLib::PACKAGE) {
+            mlua_expect!(self.disable_c_modules(), "Error during disabling C modules");
         }
+        mlua_expect!(self.extra.lock(), "extra is poisoned").libs |= libs;
+
+        res
     }
 
     /// Consumes and leaks `Lua` object, returning a static reference `&'static Lua`.
@@ -741,7 +766,7 @@ impl Lua {
         }
     }
 
-    /// Creates and returns a new table.
+    /// Creates and returns a new empty table.
     pub fn create_table(&self) -> Result<Table> {
         unsafe {
             let _sg = StackGuard::new(self.state);
@@ -755,6 +780,21 @@ impl Lua {
         }
     }
 
+    /// Creates and returns a new empty table, with the specified capacity.
+    /// `narr` is a hint for how many elements the table will have as a sequence;
+    /// `nrec` is a hint for how many other elements the table will have.
+    /// Lua may use these hints to preallocate memory for the new table.
+    pub fn create_table_with_capacity(&self, narr: c_int, nrec: c_int) -> Result<Table> {
+        unsafe {
+            let _sg = StackGuard::new(self.state);
+            assert_stack(self.state, 4);
+            protect_lua_closure(self.state, 0, 1, |state| {
+                ffi::lua_createtable(state, narr, nrec)
+            })?;
+            Ok(Table(self.pop_ref()))
+        }
+    }
+
     /// Creates a table and fills it with values from an iterator.
     pub fn create_table_from<'lua, K, V, I>(&'lua self, cont: I) -> Result<Table<'lua>>
     where
@@ -918,6 +958,7 @@ impl Lua {
     /// [`Thread`]: struct.Thread.html
     /// [`AsyncThread`]: struct.AsyncThread.html
     #[cfg(feature = "async")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "async")))]
     pub fn create_async_function<'lua, 'callback, A, R, F, FR>(
         &'lua self,
         func: F,
@@ -960,7 +1001,19 @@ impl Lua {
     where
         T: 'static + MaybeSend + UserData,
     {
-        unsafe { self.make_userdata(data) }
+        unsafe { self.make_userdata(UserDataWrapped::new(data)) }
+    }
+
+    /// Create a Lua userdata object from a custom serializable userdata type.
+    ///
+    /// Requires `feature = "serialize"`
+    #[cfg(feature = "serialize")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "serialize")))]
+    pub fn create_ser_userdata<T>(&self, data: T) -> Result<AnyUserData>
+    where
+        T: 'static + MaybeSend + UserData + Serialize,
+    {
+        unsafe { self.make_userdata(UserDataWrapped::new_ser(data)) }
     }
 
     /// Returns a handle to the global environment.
@@ -1021,6 +1074,7 @@ impl Lua {
     ///
     /// [`scope`]: #method.scope
     #[cfg(feature = "async")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "async")))]
     pub fn async_scope<'lua, 'scope, R, F, FR>(
         &'lua self,
         f: F,
@@ -1481,7 +1535,7 @@ impl Lua {
         let no_methods = methods.methods.is_empty();
 
         if no_methods {
-            init_userdata_metatable::<RefCell<T>>(self.state, -1, None)?;
+            init_userdata_metatable::<UserDataCell<T>>(self.state, -1, None)?;
         } else {
             protect_lua_closure(self.state, 0, 1, |state| {
                 ffi::lua_newtable(state);
@@ -1502,21 +1556,49 @@ impl Lua {
                 })?;
             }
 
-            init_userdata_metatable::<RefCell<T>>(self.state, -2, Some(-1))?;
+            init_userdata_metatable::<UserDataCell<T>>(self.state, -2, Some(-1))?;
             ffi::lua_pop(self.state, 1);
         }
 
-        let id = protect_lua_closure(self.state, 1, 0, |state| {
-            ffi::luaL_ref(state, ffi::LUA_REGISTRYINDEX)
+        let (ptr, id) = protect_lua_closure(self.state, 1, 0, |state| {
+            let ptr = ffi::lua_topointer(state, -1) as isize;
+            let id = ffi::luaL_ref(state, ffi::LUA_REGISTRYINDEX);
+            (ptr, id)
         })?;
 
-        mlua_expect!(self.extra.lock(), "extra is poisoned")
-            .registered_userdata
-            .insert(TypeId::of::<T>(), id);
+        let mut extra = mlua_expect!(self.extra.lock(), "extra is poisoned");
+        extra.registered_userdata.insert(TypeId::of::<T>(), id);
+        extra.registered_userdata_mt.insert(ptr);
 
         Ok(id)
     }
 
+    // Pushes a LuaRef value onto the stack, checking that it's any registered userdata
+    // Uses 2 stack spaces, does not call checkstack
+    #[cfg(feature = "serialize")]
+    pub(crate) unsafe fn push_userdata_ref(&self, lref: &LuaRef) -> Result<()> {
+        self.push_ref(lref);
+        if ffi::lua_getmetatable(self.state, -1) == 0 {
+            Err(Error::UserDataTypeMismatch)
+        } else {
+            // Check that this is our metatable
+            let ptr = ffi::lua_topointer(self.state, -1) as isize;
+            let extra = mlua_expect!(self.extra.lock(), "extra is poisoned");
+            if !extra.registered_userdata_mt.contains(&ptr) {
+                // Maybe UserData destructed?
+                get_destructed_userdata_metatable(self.state);
+                if ffi::lua_rawequal(self.state, -1, -2) == 1 {
+                    Err(Error::UserDataDestructed)
+                } else {
+                    Err(Error::UserDataTypeMismatch)
+                }
+            } else {
+                ffi::lua_pop(self.state, 1);
+                Ok(())
+            }
+        }
+    }
+
     // Creates a Function out of a Callback containing a 'static Fn.  This is safe ONLY because the
     // Fn is 'static, otherwise it could capture 'callback arguments improperly.  Without ATCs, we
     // cannot easily deal with the "correct" callback type of:
@@ -1590,7 +1672,12 @@ impl Lua {
         'lua: 'callback,
     {
         #[cfg(any(feature = "lua54", feature = "lua53", feature = "lua52"))]
-        self.load_from_std_lib(StdLib::COROUTINE)?;
+        {
+            let libs = mlua_expect!(self.extra.lock(), "extra is poisoned").libs;
+            if !libs.contains(StdLib::COROUTINE) {
+                self.load_from_std_lib(StdLib::COROUTINE)?;
+            }
+        }
 
         unsafe extern "C" fn call_callback(state: *mut ffi::lua_State) -> c_int {
             callback_error(state, |nargs| {
@@ -1717,7 +1804,7 @@ impl Lua {
         .into_function()
     }
 
-    pub(crate) unsafe fn make_userdata<T>(&self, data: T) -> Result<AnyUserData>
+    pub(crate) unsafe fn make_userdata<T>(&self, data: UserDataWrapped<T>) -> Result<AnyUserData>
     where
         T: 'static + UserData,
     {
@@ -1725,7 +1812,7 @@ impl Lua {
         assert_stack(self.state, 4);
 
         let ud_index = self.userdata_metatable::<T>()?;
-        push_userdata::<RefCell<T>>(self.state, RefCell::new(data))?;
+        push_userdata::<UserDataCell<T>>(self.state, RefCell::new(data))?;
 
         ffi::lua_rawgeti(
             self.state,
@@ -1880,6 +1967,7 @@ impl<'lua, 'a> Chunk<'lua, 'a> {
     ///
     /// [`Chunk::exec`]: struct.Chunk.html#method.exec
     #[cfg(feature = "async")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "async")))]
     pub fn exec_async<'fut>(self) -> LocalBoxFuture<'fut, Result<()>>
     where
         'lua: 'fut,
@@ -1919,6 +2007,7 @@ impl<'lua, 'a> Chunk<'lua, 'a> {
     ///
     /// [`Chunk::eval`]: struct.Chunk.html#method.eval
     #[cfg(feature = "async")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "async")))]
     pub fn eval_async<'fut, R>(self) -> LocalBoxFuture<'fut, Result<R>>
     where
         'lua: 'fut,
@@ -1953,6 +2042,7 @@ impl<'lua, 'a> Chunk<'lua, 'a> {
     ///
     /// [`Chunk::call`]: struct.Chunk.html#method.call
     #[cfg(feature = "async")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "async")))]
     pub fn call_async<'fut, A, R>(self, args: A) -> LocalBoxFuture<'fut, Result<R>>
     where
         'lua: 'fut,
